#Fengyun 2H Atmospheric motion vectors import routine
#Output table columns need further specifications, now only called a, b, c, etc
#Import of the IR1_006 and IR3_006

#Further processing in ILWIS386: Convert table to point map and use table domain, display pointmap and use symbol as arrow option 
#Wind direction is column B and wind speed is column C

       
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import subprocess
import shutil
import glob
import gzip
import zipfile
from threading import Thread

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
                
        os.chdir(WorkDir)
        
        #filename = Z_SATE_C_BABJ_20250428060000_O_FY2H_AMV_IR1_006_OTG.AWX.gz and Z_SATE_C_BABJ_20250428060000_O_FY2H_AMV_IR3_006_OTG.AWX.gz
        for filename in glob.glob(os.path.join(GNCDir+'/Z_SATE_C_BABJ_'+(InFileName)+'00_O_FY2H_AMV_IR*_006_OTG.AWX.gz')):
            #print(filename)
            shutil.copy(filename, WorkDir)

        progress['value'] = 30  

        for fnames in glob.glob(os.path.join(WorkDir+'/Z_SATE_C_BABJ_'+(InFileName)+'00_O_FY2H_AMV_IR*_006_OTG.AWX.gz')):
            file_in = fnames
            fname = file_in.rsplit('.',1)[0] #get file name without gz extension
            with gzip.open(file_in,"rb") as f_in, open(fname,"wb") as f_out:
                shutil.copyfileobj(f_in, f_out)

        old_path = WorkDir+'/Z_SATE_C_BABJ_'+(InFileName)+'00_O_FY2H_AMV_IR1_006_OTG.AWX'
        new_path = WorkDir+'/'+OutFileName+'_IR1_006.AWX'
        # Rename the file
        os.rename(old_path, new_path)

        old_path = WorkDir+'/Z_SATE_C_BABJ_'+(InFileName)+'00_O_FY2H_AMV_IR3_006_OTG.AWX'
        new_path = WorkDir+'/'+OutFileName+'_IR3_006.AWX'
        # Rename the file
        os.rename(old_path, new_path)
        
        list = glob.glob(WorkDir+'/'+OutFileName+'_IR?_006.AWX')
        list = [elem[len(WorkDir)+1:] for elem in list]
        
        progress['value'] = 60   
        
        #redirect popup MSDOS window when executing the sub process
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW   

        progress['value'] = 80        
        
        for elem in list:
            decode = subprocess.run([gnc_path+'/extern/FY_AMV/AmvToIlwisTable.exe',elem, elem+'1'], startupinfo=si)
      
        list = glob.glob(WorkDir+'/'+(OutFileName)+'_IR?_006.AWX')
        list = [elem[len(WorkDir)+1:] for elem in list]
        for elem in list:
            os.remove(WorkDir+'/'+elem)
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported FY2H - AMV image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 620, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/FY2H_AMV.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About FY2H - AMV')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""FY2H AMV background:

FY2H IR1 and IR3 channels derived atmospheric motion vectors. ILWIS format output table columns need further specifications, now only called a, b, c, etc. Further processing in ILWIS386: Convert table to point map and use table domain, display pointmap and use symbol as arrow option. Wind direction is column B and wind speed is column C.

File name convertion: Z_SATE_C_BABJ_20250428060000_O_FY2H_AMV_IR1_006_OTG.AWX.gz and Z_SATE_C_BABJ_20250428060000_O_FY2H_AMV_IR3_006_OTG.AWX.gz
Number of segments: 1 body segment
Temporal Resolution: 6 hour, at 06:00,12:00,18:00 and 00:00 UTC
Initial file format: AWX

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Fengyun 2H AMV import routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""FY2H AMV
Typical filename: Z_SATE_C_BABJ_20250428060000_O_FY2H_AMV_IR1_006_OTG.AWX.gz""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About AMV", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



