# GOES West (currently GOES 18) GML
# requires start time - in 10 minutes intervals, e.g. yyyymmddhhmm, last mm  = 00, 10, 20, 30, 40, 50
# retrieves from date string doy number
# extract from *.tgz and *.tar the 15 *.nc files per 5 minutes
# each *.nc file has 20 seconds duration
# loop through 10 minutes GLM files (30) and retrieve lat, lon and energy
# each file contains 20 seconds of data 

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style

import tarfile
import shutil
import sys, os 
import glob
import subprocess
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import geopandas as gpd
from shapely.geometry import Point
from netCDF4 import Dataset
from threading import Thread


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        
        time_stamp = InFileName
        yyyy = int(time_stamp[0:4])
        mm = int(time_stamp[4:6])
        dd = int(time_stamp[6:8])
        tt = int(time_stamp[8:11]) #only first 3 digits are used here!

        doy = datetime(yyyy, mm, dd).timetuple().tm_yday
        #print(doy)
        #print(tt)

        doys = "{:03d}".format(doy)
        #print(doys)

        os.chdir(WorkDir)
        
        progress['value'] = 10
        
        #loop through gtz and nc files for 10 minutes GLM files (30) and retrieve lat, lon and energy
        for fnames in glob.glob(os.path.join(GNCDir+'/OR_GLM-L2-LCFA_G18_s'+str(yyyy)+str(doys)+str(tt)+'*.tgz')):
            shutil.copy(fnames, WorkDir)
            
        #extract files from *.tgz 
        glm_list = glob.glob(WorkDir+'/OR_GLM-L2-LCFA_G18_s'+str(yyyy)+str(doys)+str(tt)+'*.tgz')
        glm_list = [elem[len(WorkDir)+1:] for elem in glm_list]
        for elem in glm_list:
            print(elem)
            
            
        def unzip_tgz_files(tgz_files, output_dir):
            for tgz_file in tgz_files:
                try:
                    # Ensure the output directory exists
                    os.makedirs(output_dir, exist_ok=True)
            
                    # Open the .tgz file
                    with tarfile.open(tgz_file, 'r:gz') as tar_ref:
                        # Extract all contents to the output directory
                        tar_ref.extractall(output_dir)
                        print(f"Extracted {tgz_file} to {output_dir}")
                except Exception as e:
                    print(f"Failed to unzip {tgz_file}: {e}")    
            
        unzip_tgz_files(glm_list, WorkDir)    

        #create list of NC files  
        glm_list1 = glob.glob(WorkDir+'/OR_GLM-L2-LCFA_G18_s'+str(yyyy)+str(doys)+str(tt)+'*.nc')
        glm_list1 = [elem[len(WorkDir)+1:] for elem in glm_list1]
        for elem in glm_list1:
            print(elem)       
        
        #process GLM_latitude 
        GLM_latitude = np.array([])

        for GLM in (glm_list1):
            #print('now processed = ', GLM.rsplit('_')[-3][1:14])
            GLM = Dataset(GLM, 'r')
            latitude = GLM.variables['flash_lat'][:].data
            GLM_latitude = np.append(GLM_latitude, latitude)

        #print(GLM_latitude)
        
        progress['value'] = 30
        
        #process longitude
        GLM_longitude = np.array([])

        for GLM in (glm_list1):
            #print ('now processed = ',GLM)
            GLM = Dataset(GLM, 'r')
            longitude = GLM.variables['flash_lon'][:].data
            GLM_longitude = np.append(GLM_longitude, longitude)

        #print(GLM_longitude)
        
        
        #process energy
        GLM_energy = np.array([])
        
        for GLM in (glm_list1):
            #print ('now processed = ',GLM)
            GLM = Dataset(GLM, 'r')
            energy = GLM.variables['flash_energy'][:].data
            GLM_energy = np.append(GLM_energy, energy)

        #print(GLM_energy)
        
        progress['value'] = 60
        dataset = pd.DataFrame({'LON':GLM_longitude, 'LAT':GLM_latitude,'Fl_ENERGY':GLM_energy})


        #create shape file
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset['LON']=dataset['LON'].astype(float)
        dataset['LAT']=dataset['LAT'].astype(float)
        dataset['Fl_ENERGY']=dataset['Fl_ENERGY'].astype(float)
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset['LON'], dataset['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset, geometry=geometry)
        
        progress['value'] = 80

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')

        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported GOES West GLM image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 400, height = 331)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/g18_glm.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About GOES GML')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""GOES GML background:

The Geostationary Lightning Mapper (GLM) is a satellite-borne single channel, near-infrared optical transient detector that has been placed on the current GOES series of satellites in a geostationary orbit. This orbital position allows for GLM to provide lightning detection at a rate never before obtained from space. GLM detects all forms of lightning during both day and night, continuously, with a high spatial resolution and detection efficiency.

File name convertion: OR_GLM-L2-LCFA_G18_s*.nc
Number of files: loops through 10 minutes of GLM files (30) and retrieve lat, lon and energy
Temporal resolution: Every 10 minutes, starting from 00:00 UTC
Initial file format: NetCDF

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("GOES WEST GML import routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""GOES EAST ABI 
Typical filename: OR_GLM-L2-LCFA_G18_s*.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time (yyyymmddhhmm):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename prefix:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About GOES-GML", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



