#LSASAF Full Disk ET daily import routine
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob
import bz2

import ilwis
import netCDF4 as nc
from threading import Thread
import subprocess
import numpy as np


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp_in = InFileName
        timeA = time_stamp_in[:8]
        timeB = time_stamp_in[8:12]
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filename expected =  S-LSA_-HDF5_LSASAF_MSG_ET_MSG-Disk_202504111200.bz2
        for filename in glob.glob(os.path.join(GNCDir+'/S-LSA_-HDF5_LSASAF_MSG_ET_MSG-Disk_'+ InFileName +'.bz2')):
            shutil.copy(filename, WorkDir)

        progress['value'] = 20

        for fnames in glob.glob(os.path.join(WorkDir+'/S-LSA_-HDF5_LSASAF_MSG_ET_MSG-Disk_'+ InFileName +'.bz2')):
            file_in = fnames

        fname = file_in.rsplit('.',1)[0] #get file name without bz2 extension
        with bz2.open(file_in,"rb") as f_in, open(fname,"wb") as f_out:
            f_out.write(f_in.read()) 

        progress['value'] =40
        
        #load ET dataset
        ds = nc.Dataset(fname)
        data = ds['ET']

        #create numpy array from nc variable
        e = data[:].flatten()
        
        progress['value'] = 60
        
        # create output map in ILWIS format
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +h=35785831 +a=6378137 +rf=298.257223563,envelope=-5570248.4773 5570248.4773 5567248.0742 -5567248.0742,gridsize=3712 3712,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0.0, 100000000.0, 0.000))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(3712,3712,1))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        progress['value'] = 80
        
        rcNew.array2raster(e)
        rc_cor = ilwis.do('mapcalc', 'iff(@1<=-1, ?, @1/10000)',rcNew) 
        rc_cor.store(OutFileName+'.mpr')
              

        #Other redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized ET 30 minutes image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 400, height = 401)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/LSASAF_ET.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About ET')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""ET background:

Evapotranspiration (ET) accounts for the flux of water evaporated at the Earth-atmosphere interface (from soil, water bodies and interception) and transpired by vegetation through stomata in its leaves as a consequence of photosynthetic processes. ET is an important component of the water cycle and it is associated with the latent heat flux (LE), a key link between the energy and water cycles. In other words, LE represents the energy needed for the ET process. Evaluating energy fluxes at the Earth surface is of great importance in many disciplines like weather forecasting, global climate monitoring, water management, agriculture and ecology.

File name convertion: S-LSA_-HDF5_LSASAF_MSG_ET_MSG-Disk_202504111200.bz2 
Number of segments: 1 file
Temporal resolution: 1 per 30 minutes
Initial file format: nc format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("LSA SAF ET")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""LSA SAF ET import routine   
Typical filename: S-LSA_-HDF5_LSASAF_MSG_ET_MSG-Disk_202504111200.bz2 """)

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About ET", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


