#MPEF MSG Atmospheric Motion Vectors (AMV) import routine

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob

from sklearn import preprocessing
from pybufrkit.decoder import Decoder
from pybufrkit.mdquery import MetadataExprParser, MetadataQuerent
from pybufrkit.dataquery import NodePathParser, DataQuerent
from threading import Thread
import subprocess

import geopandas as gpd
from shapely.geometry import Point

import numpy as np
import pandas as pd


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        #ilwis.setWorkingCatalog(WorkDir)
        
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+ASR_C_EUMG_'+str(time_stamp)+'00_*.bin')):
            shutil.copy(filename, WorkDir)
            
        for fnames in glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+ASR_C_EUMG_'+(time_stamp)+'00_*.bin')):
            file_in = fnames
            #print(file_in)
        
        progress['value'] = 20
        
        #redirect popup MSDOS window when executing the sub process
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        
        #split all messages contained in BUFR file into seperate messages
        decode = subprocess.run(['pybufrkit', 'split', file_in], startupinfo=si)
        
        #read all messages and append 
        messages = []
        decoder = Decoder()
        path = file_in
        filelist = glob.glob(path + '.*')
        for file in filelist:
            with open((file), 'rb') as ins:
                bufr_message = decoder.process(ins.read())
                messages.append(bufr_message)
        
        # Query the metadata
        n_subsets = MetadataQuerent(MetadataExprParser()).query(bufr_message, '%n_subsets')
        print(n_subsets)
        
        #Relevant coordinate variables are: 14-latitude (code 005001), 15-longitude (code 006001)
        #Relevant Brightness Temperature parameters are: brightness temperature (code 012063)
        #other parameters extracted are: Amount of segment cloud free (code 020082)


        progress['value'] = 20
        
        #ASR Brightness Temperature BUFR Data retrieved
        lats = np.array([])
        lons = np.array([])


        bt1 = np.array([])
        bt2 = np.array([])
        bt3 = np.array([])
        bt4 = np.array([])
        bt5 = np.array([])
        bt6 = np.array([])
        bt7 = np.array([])
        bt8 = np.array([])
        bt9 = np.array([])
        bt10 = np.array([])

        for  message in messages:
            LAT_result = DataQuerent(NodePathParser()).query(message, '005001')
            lats=np.concatenate((lats, np.array([LAT_result.results[key][0] for key in LAT_result.results.keys()])))
            LON_result = DataQuerent(NodePathParser()).query(message, '006001')
            lons=np.concatenate((lons,np.array([LON_result.results[key][0] for key in LON_result.results.keys()])))

            BT3 = DataQuerent(NodePathParser()).query(message, '012063')
            bt3 = np.concatenate((bt3,np.array([BT3.results[key][0][3][0] for key in BT3.results.keys()])))
            BT4 = DataQuerent(NodePathParser()).query(message, '012063')
            bt4 = np.concatenate((bt4,np.array([BT4.results[key][0][4][0] for key in BT4.results.keys()])))

            BT5 = DataQuerent(NodePathParser()).query(message, '012063')
            bt5 = np.concatenate((bt5,np.array([BT5.results[key][0][5][0] for key in BT5.results.keys()])))
            BT6 = DataQuerent(NodePathParser()).query(message, '012063')
            bt6 = np.concatenate((bt6,np.array([BT6.results[key][0][6][0] for key in BT6.results.keys()])))

            BT7 = DataQuerent(NodePathParser()).query(message, '012063')
            bt7 = np.concatenate((bt7,np.array([BT7.results[key][0][7][0] for key in BT7.results.keys()])))
            BT8 = DataQuerent(NodePathParser()).query(message, '012063')
            bt8 = np.concatenate((bt8,np.array([BT8.results[key][0][8][0] for key in BT8.results.keys()])))

            BT9 = DataQuerent(NodePathParser()).query(message, '012063')
            bt9 = np.concatenate((bt9,np.array([BT9.results[key][0][9][0] for key in BT9.results.keys()])))
            BT10 = DataQuerent(NodePathParser()).query(message, '012063')
            bt10 = np.concatenate((bt10,np.array([BT10.results[key][0][10][0] for key in BT10.results.keys()])))
        
        progress['value'] = 40
        
        # convert numpy array to pandas dataframe
        dataset_BT = pd.DataFrame({'LAT':lats, 'LON':lons, 'BT3':bt3, 'BT4':bt4, 'BT5':bt5, 'BT6':bt6, 'BT7':bt7, 'BT8':bt8, 'BT9':bt9, 'BT10':bt10})

        
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset_BT['TB3']=dataset_BT['BT3'].astype(float)
        dataset_BT['TB4']=dataset_BT['BT4'].astype(float)
        dataset_BT['TB5']=dataset_BT['BT5'].astype(float)
        dataset_BT['TB6']=dataset_BT['BT6'].astype(float)
        dataset_BT['TB7']=dataset_BT['BT7'].astype(float)
        dataset_BT['TB8']=dataset_BT['BT8'].astype(float)
        dataset_BT['TB9']=dataset_BT['BT9'].astype(float)
        dataset_BT['TB10']=dataset_BT['BT10'].astype(float)

        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset_BT['LON'], dataset_BT['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset_BT, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'_BT.shp')
        
        #ASR Cloud Free BUFR Data retrieved
        lats = np.array([])
        lons = np.array([])
        cf = np.array([]) #cloud free

        for  message in messages:
            LAT_result = DataQuerent(NodePathParser()).query(message, '005001')
            lats=np.concatenate((lats, np.array([LAT_result.results[key][0] for key in LAT_result.results.keys()])))
            LON_result = DataQuerent(NodePathParser()).query(message, '006001')
            lons=np.concatenate((lons,np.array([LON_result.results[key][0] for key in LON_result.results.keys()])))
            CF = DataQuerent(NodePathParser()).query(message, '020082')
            cf = np.concatenate((cf,np.array([CF.results[key][0] for key in CF.results.keys()])))
            
        dataset_CF = pd.DataFrame({'LAT':lats, 'LON':lons, 'CF':cf}) 
        #print(dataset_CF.dtypes)
        
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset_CF['CF']=dataset_CF['CF'].astype(float)

        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset_CF['LON'], dataset_CF['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset_CF, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'_CF.shp')
      
        #Redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG ASR Cloud Free image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 407, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MPEF_MSG0_ASR_CF.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About AMV')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MPEF MSG ASR background:

All-Sky Radiances - MSG - 0 degree. The ASR product contains information on mean brightness temperatures (16x16 pixels so around 50km at nadir) from all thermal (e.g. infrared and water vapour) channels. It includes both clear and cloudy sky brightness temperatures. The product provides brightness temperatures for the MSG infrared channels averaged over all pixels within a processing segment and in six separate categories: all pixels, clear pixels, cloudy pixels, and low-, mid- and high-level cloud pixels, provided in 2 shape files (with suffix *_BT and *_CF) for brightness temperature and % cloud free respectively.

File name convertion: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<number>ASR_C_EUMG_<date stamp>00_*.bin
Number of segments: 1 file
Temporal resolution: Every 1 hour, starting from 00:45 UTC
Initial file format: bufr format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("MPEF MSG ASR    (Import routine takes some time - be patient!)")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""MPEF MSG All Sky Radiances import routine   
Typical filename: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<no>+ASR_C_EUMG_<date>00_<version>.bin""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About ASR", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


