#MPEF MSG Optimum Cloud Analysis (OCAE) import routine
#Import of selected variables
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob

from threading import Thread
import subprocess

from osgeo import gdal
import ilwis
import numpy as np


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+OCAE_C_EUMG_'+str(time_stamp)+'00_*.bin')):
            shutil.copy(filename, WorkDir) 
        
        progress['value'] = 20

        for fnames in glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+OCAE_C_EUMG_'+str(time_stamp)+'00_*.bin')):
            file_in = fnames
            #print(file_in)

        progress['value'] =40
        
        #load OCAE data using gdal
        #load layer 0: Cloud Phase, 111 = Single layer Water cloud, 112 = Single layer Ice cloud, 24 = ??        
        dataset = gdal.Open(file_in)
        data_array = dataset.ReadAsArray().astype(float)
        L0 = np.array([data_array[0]])
        L0 = L0.reshape(data_array.shape[2],data_array.shape[1],1)
        
        #load layer 2: Upper Layer Cloud Optical Depth, - Unitless       
        dataset = gdal.Open(file_in)
        data_array = dataset.ReadAsArray().astype(float)
        L2 = np.array([data_array[2]])
        L2 = L2.reshape(data_array.shape[2],data_array.shape[1],1)
        
        #load layer 3: Upper Layer Cloud Top Pressure, Pa      
        dataset = gdal.Open(file_in)
        data_array = dataset.ReadAsArray().astype(float)
        L3 = np.array([data_array[3]])
        L3 = L3.reshape(data_array.shape[2],data_array.shape[1],1)
        
        #load layer 4: Upper Layer Cloud Effective Radius, m     
        dataset = gdal.Open(file_in)
        data_array = dataset.ReadAsArray().astype(float)
        L4 = np.array([data_array[4]])
        L4 = L4.reshape(data_array.shape[2],data_array.shape[1],1)
        
        data = np.array((L0, L2, L3, L4))
               
        progress['value'] = 60
        
        # create output map in ILWIS format
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +h=35785831 +a=6378137 +rf=298.257223563,envelope=-5570248.4773 5570248.4773 5567248.0742 -5567248.0742,gridsize=3712 3712,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-1000000.000, 1000000.000, 0.0000000001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(3712,3712,4))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        rcNew.array2raster(data.flatten())
        
        progress['value'] = 80
        
        rcNew = ilwis.do('mapcalc', 'iff(@1==9999,?,@1)', rcNew)
        rcNew.store(OutFileName+'.mpl')
        

        #Redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG OCAE (Cloud Top pressure) image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 400, height = 398)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MPEF_MSG0_OCAE.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About CTH')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MPEF MSG OCAE background:

The Optimal Cloud Analysis is an algorithm to extract the principal cloud parameters, such as 1:Phase, 2:Optical Depth (Unitless), 3:Pressure (Pa) and 4:Effective Radius (m) using an optimal estimation method and simultaneously all the SEVIRI spectral measurements. 
Cloud Phase values: 111 = Single layer Water cloud, 112 = Single layer Ice cloud, 24 = unknown 

File name convertion: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<number>OCAE_C_EUMG_<date stamp>00_*.bin
Number of segments: 1 file
Temporal resolution: Every 15 minutes, starting from 00:00 UTC
Initial file format: grib format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("MPEF MSG OCAE")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""MPEF MSG Optimum Cloud Analysis import routine   
Typical filename: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<no>+OCAE_C_EUMG_<date>00_<version>.bin""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About OCAE", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


