#MPEF MSG Atmospheric Motion Vectors (AMV) import routine IODC

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob

from sklearn import preprocessing
from pybufrkit.decoder import Decoder
from pybufrkit.mdquery import MetadataExprParser, MetadataQuerent
from pybufrkit.dataquery import NodePathParser, DataQuerent
from threading import Thread
import subprocess

import geopandas as gpd
from shapely.geometry import Point
import numpy as np
import pandas as pd


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        
        for filename in glob.glob(os.path.join(GNCDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+AMV_C_EUMG_'+str(time_stamp)+'00_*.bin')):
            shutil.copy(filename, WorkDir)
            
        for fnames in glob.glob(os.path.join(WorkDir+'/W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET'+'*+AMV_C_EUMG_'+(time_stamp)+'00_*.bin')):
            file_in = fnames
            #print(file_in)
        
        progress['value'] = 20
        
        #redirect popup MSDOS window when executing the sub process
        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        
        #split all messages contained in BUFR file into seperate messages
        decode = subprocess.run(['pybufrkit', 'split', file_in], startupinfo=si)
        
        #read all messages and append 
        messages = []
        decoder = Decoder()
        path = file_in
        filelist = glob.glob(path + '.*')
        for file in filelist:
            with open((file), 'rb') as ins:
                bufr_message = decoder.process(ins.read())
                messages.append(bufr_message)
        
        # Query the metadata
        n_subsets = MetadataQuerent(MetadataExprParser()).query(bufr_message, '%n_subsets')
        print(n_subsets)
        
        #Relevant coordinate variables are: 14-latitude (code 005001), 15-longitude (code 006001)
        #Relevant parameters are: 29-Pressure (code 007004), V-Component (011004), U-Component (011003) and Pressure (007004)
        #other parameters extracted are: 30-Wind_direction (code 011001), 31-Wind_speed (code 011002) and 34-Temperature (code 012001)


        progress['value'] = 20
        
        from pybufrkit.dataquery import NodePathParser, DataQuerent
        lats = np.array([])
        lons = np.array([])
        winddirs = np.array([])
        windsps = np.array([])
        windvs = np.array([])
        windus = np.array([])
        press = np.array([])
        tempers = np.array([])

        for  message in messages:
            LAT_result = DataQuerent(NodePathParser()).query(message, '005001')
            lats=np.concatenate((lats, np.array([LAT_result.results[key][0] for key in LAT_result.results.keys()])))
            LON_result = DataQuerent(NodePathParser()).query(message, '006001')
            lons=np.concatenate((lons,np.array([LON_result.results[key][0] for key in LON_result.results.keys()])))
            WINDDir_result = DataQuerent(NodePathParser()).query(message, str('011001'))
            winddirs=np.concatenate((winddirs,np.array([WINDDir_result.results[key][0] for key in WINDDir_result.results.keys()])))            
            WINDSP_result = DataQuerent(NodePathParser()).query(message, str('011002'))
            windsps=np.concatenate((windsps,np.array([WINDSP_result.results[key][0] for key in WINDSP_result.results.keys()]))) 
            WINDV_result = DataQuerent(NodePathParser()).query(message, str('011004'))
            windvs=np.concatenate((windvs,np.array([WINDV_result.results[key][0] for key in WINDV_result.results.keys()])))
            WINDU_result = DataQuerent(NodePathParser()).query(message, str('011003'))
            windus=np.concatenate((windus,np.array([WINDU_result.results[key][0] for key in WINDU_result.results.keys()])))
            PRESSURE_result = DataQuerent(NodePathParser()).query(message, str('007004'))
            press=np.concatenate((press,np.array([PRESSURE_result.results[key][0] for key in PRESSURE_result.results.keys()])))
            TEMPERATURE_result = DataQuerent(NodePathParser()).query(message, str('012001'))
            tempers=np.concatenate((tempers,np.array([TEMPERATURE_result.results[key][0] for key in TEMPERATURE_result.results.keys()])))
            
        
        progress['value'] = 40
        
        # convert numpy array to pandas dataframe
        dataset = pd.DataFrame({'LAT': lats, 'LON': lons, 'WIND_DIR': winddirs, 'WIND_SPEED':windsps, 'WINDV': windvs, 'WINDU': windus, 'PRESSURE':press, 'TEMPERATURE': tempers})
        #dataset       
 

        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset['WIND_DIR']=dataset['WIND_DIR'].astype(float)
        dataset['WIND_SPEED']=dataset['WIND_SPEED'].astype(float)
        dataset['WINDV']=dataset['WINDV'].astype(float)
        dataset['WINDU']=dataset['WINDU'].astype(float)
        dataset['PRESSURE']=dataset['PRESSURE'].astype(float)
        dataset['TEMPERATURE']=dataset['TEMPERATURE'].astype(float)
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset['LON'], dataset['LAT'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset, geometry=geometry)

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp') 
 
      
        #Redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG AMV image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 650, height = 390)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MPEF_MSGRSS_AMV.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About AMV')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MPEF MSG AMV background:

Atmospheric Motion Vectors at all heights below the tropopause, derived from 5 channels (0.8, 6.2, 7.3, 10.8 and HVR channels). Vectors are derived by tracking the motion of clouds and other atmospheric constituents as water vapour patterns. The initial resolution is a 24 pixels grid (HRV 12 high res. pixels), but as the algorithm tries to adjust the position to the point of the maximum contrast (typically cloud edges), the end resolution varies. The height assignment of the AMVs is calculated using the Cross-Correlation Contribution (CCC) function to determine the pixels that contribute the most to the vectors. An AMV product contains between 30 000 and 50 000 vectors depending of the time of the day. Output routine is a shape file.

File name convertion: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<number>AMV_C_EUMG_<date stamp>00_*.bin
Number of segments: 1 file
Temporal resolution: Every 20 minutes, starting from 00:10 UTC
Initial file format: bufr format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("MPEF MSG AMV    (Import routine takes some time - be patient!)")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""MPEF MSG Atmospheric Motion Vector import routine   
Typical filename: W_XX-EUMETSAT-Darmstadt,SING+LEV+SAT,MET<no>+AMV_C_EUMG_<date>00_<version>.bin""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About AMV", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


