#Meteosat Second Generation Normal Resolution Visualizations
#Create a Daytime MicroPhysics Composite
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
from threading import Thread
import subprocess

from satpy.scene import Scene
from satpy.writers import get_enhanced_image
import glob
from osgeo import gdal
import numpy as np
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp = InFileName
        year = time_stamp[0:4]
        month = time_stamp[4:6]
        day = time_stamp[6:8]
        timeUTC = time_stamp[8:12]
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        for filename in glob.glob(os.path.join(GNCDir+'/'+'H*'+(time_stamp)+'*')):
            shutil.copy(filename, WorkDir)
        shutil.copy(gnc_path + '/extern/MSG_decomp/xRITDecompress.exe', WorkDir)
        
        progress['value'] = 20

        list = glob.glob(WorkDir+'/H*'+(time_stamp)+'*')
        list = [elem[len(WorkDir)+1:] for elem in list]

        si = subprocess.STARTUPINFO()
        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        for elem in list:
            decode = subprocess.run(['xRITDecompress.exe',elem], startupinfo=si)
        
        #load data for selected color visualization
        fnames = glob.glob(WorkDir+'/H*'+(time_stamp)+'*__')
        scn = Scene(reader='seviri_l1b_hrit', filenames=fnames)

        progress['value'] =40

        #load day microphysics composite
        scn.load(["day_microphysics"], upper_right_corner='NE')
        scn.save_dataset('day_microphysics', filename=WorkDir+'/'+'MSG_DMP.tif')
        
        progress['value'] = 60
        
        # create output map in ILWIS format using gdal-translate
        output_file = WorkDir+'/'+OutFileName+'.mpl'

        gdal_in = gdal.Open(WorkDir+'/MSG_DMP.tif')
        data_in = gdal_in.ReadAsArray().astype(np.ubyte)

        dst_ds = gdal.Translate(output_file, gdal_in, bandList=[3,2,1], format='ILWIS')
        
        dst_ds = None
        gdal_in = None
        scn = None
        
        progress['value'] = 80
        
        for file in fnames:
            os.remove(file)
        ornames = glob.glob(os.path.join(WorkDir+'/'+'H*'+(time_stamp)+'*'))
        for file in ornames:
            os.remove(file)
        
        
        #redundant files to be deleted 
        if os.path.exists(WorkDir + '/xRITDecompress.exe'):
            os.remove (WorkDir + '/xRITDecompress.exe')
        if os.path.exists(WorkDir+'/MSG_DMP.tif'):
            os.remove (WorkDir+'/MSG_DMP.tif')
        if os.path.exists(output_file +'.aux.xml'):
            os.remove (output_file +'.aux.xml')

        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG Daytime MicroPhysics image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 500, height = 313)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MSG_DMP.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MSG')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MSG background:

MSG is a spinning stabilised twelve channel imager, called SEVIRI (Spinning Enhanced Visible and Infrared Imager) and observes the full disk of the Earth with a repeat cycle of 15 minutes. SEVIRI has eight channels in the thermal infrared (IR) at 3.9, 6.2, 7.3, 8.7, 9.7, 10.8, 12.0 and 13.4 µm, three channels in the solar spectrum at 0.6, 0.8, 1.6 µm and a broad-band high resolution visible channel. The high resolution visible channel has a spatial resolution of 1 km at nadir. The corresponding values for the eight thermal IR and the other three solar channels are  3 km at nadir.

File name convertion: H-000-MSG?__-MSG?________-'spectral channel'___-00000?___-'date'-C_ 
Number of segments: 112 body segments and Epi / Pro file
Temporal resolution: Every 15 minutes, starting from 00:00 UTC
Initial file format: wavelet compressed segments

Data is provided throught EUMETCast reception and EUMDAC (online). For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Meteosat Second Generation Daytime MicroPhysics Composite Visualization routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Meteosat Second Generation SEVIRI Visualization Import Routine  
Typical filename: H-000-MSG?__-MSG?________-'spectral channel'___-00000?___-'date'-C_ """)

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About MSG", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


