#Meteosat Third Generation Lightning Imager Level 2 Accumulated Flash Area (AFA) 

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import ilwis
import datetime
import shutil
from threading import Thread
from satpy.scene import Scene
from satpy.composites import GenericCompositor
from satpy.resample import get_area_def
from satpy import find_files_and_readers
from satpy.dataset import combine_metadata
from datetime import datetime
import netCDF4
import glob
import numpy as np
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0

        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        shutil.copy(GNCDir + '/'+ InFileName, WorkDir)
        
        fnames = [WorkDir + '/' + InFileName]
        
        #read scene using satpy
        scn = Scene(reader='li_l2_nc', filenames=fnames)

        progress['value'] = 20

        #load accumulated flash area
        scn.load(["acc_flash_area"])
        values = scn['acc_flash_area'].values
        #flip UD image
        rot = np.flipud(values)
        data = np.array([rot]).flatten()
        
        progress['value'] = 60
        
        #export to ILWIS format
        #create an empty raster
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +h=35786400 +a=6378137 +rf=298.257223563,envelope=-5567999.9942 5567999.9942 5567999.9942 -5567999.9942,gridsize=5568 5568,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0.0, 500.0, 0.001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(5568,5568,1))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        #write data to raster
        rcNew.array2raster(data)
            
        #correct background and save the results to disk as an ILWIS mpr file
        rcNew = ilwis.do('mapcalc', 'iff(@1>=0,@1,?)', rcNew) # remove (satpy) nodata
        rcNew.store(WorkDir+'/'+OutFileName+'.mpr')   
        
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported MTG LI AFA image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 500, height = 321)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MTG_LI_AFA.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MTG Li LFL')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MTG LI AFA background:

I Level 2 Accumulated Flash Area (AFA) provides the user with data about flash mapping by using the area covered by the optical emission of each flash in LI Level 2 Lightning Flashes (LFL). It is important to keep in mind that each flash is treated as a flat (uniform) optical emission in this data. Accumulated Flash Area allows one to monitor the regions within a cloud top from which lightning-related optical emissions over 30 sec are emerging and accumulating and to know the number of flashes that were observed within the FCI grid pixels composing those regions. For example, from the Accumulated Flash Area, one can derive the flash rate for each pixel of the FCI 2km grid. This is a considerable improvement compared to the simple description of the flash using the variable flash_footprint available in Lightning Flashes.

File name convertion: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-AFA--FD--CHK-BODY---NC4E_C_EUMT_*.nc
Number of segments: 1 body segment
Temporal reolution: Every 30 seconds, starting from 00:00 UTC
Initial file format: NetCDF

Data is provided throught EUMETCast reception and EUMDAC (online). For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Meteosat Third Generation Lightning Imager Level 2 Accumulated Flash Area (AFA) import routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Meteosat Third Generation Lightning Imager Level 2 Accumulated Flash Area (AFA) 
Typical filename: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-AFA--FD--CHK-BODY---NC4E_C_EUMT_*.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Input Filename:", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
button_filein.pack()
button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About MTG-LI-AFA", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



