#Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL) 

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys

import pandas as pd
import datetime
import netCDF4
import geopandas as gpd
from shapely.geometry import Point
import shutil
from threading import Thread
import zipfile

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0

        os.chdir(WorkDir)

        shutil.copy(GNCDir + '/'+ InFileName, WorkDir)
        
        zip_content = None
        with zipfile.ZipFile(WorkDir+'/'+InFileName, 'r') as zip_ref:
            zip_content = zip_ref.namelist()
            zip_content = [file for file in zip_content if 'CHK-BODY' in file]
            for file in zip_content:
                zip_ref.extract(file)
        
        #ilwis.setWorkingCatalog(WorkDir)
        fname = WorkDir+'/'+ zip_content[0] #"W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--CHK-BODY--ARC-NC4E_C_EUMT_*.nc"
        li_ds = netCDF4.Dataset(fname, 'r')

        progress['value'] = 30

        #choose variables in netcdf file
        lat = li_ds['latitude'][:]
        lon = li_ds['longitude'][:]
        flash_id = li_ds['flash_id'][:]
        flash_time = li_ds['flash_time'][:]
        flash_dur = li_ds['flash_duration'][:]
        flash_fp = li_ds['flash_footprint'][:]
        rad = li_ds['radiance'][:]
        nb_events = li_ds['number_of_events'][:]
        nb_groups = li_ds['number_of_groups'][:]
        ff_conf = li_ds['flash_filter_confidence'][:]

        #The flash_time is measured as epoch time in products from Meteosat Third Generation satellites. 
        #To make it easier to work with them, we will recalculate the regular date time from it using the following function:
        def epoch_to_datetime(flash_time):
                flash_time_list = []
        
                for i in flash_time:
                    mtg_reference_datetime=datetime.datetime(2000, 1, 1, 0, 0)
                    mtg_datetime=mtg_reference_datetime + datetime.timedelta(seconds=i)
                    flash_time_list.append(mtg_datetime)

                return flash_time_list

        progress['value'] = 60

        # Building Panda dataframe
        dataset = pd.DataFrame({
                'lat': lat.flatten(),
                'lon': lon.flatten(),
                'flash_id': flash_id.flatten(),
                'flash_time': epoch_to_datetime(flash_time.flatten()),
                'flash_dur': flash_dur.flatten(),
                'flash_fp': flash_fp.flatten(),
                'rad_scale': rad.flatten(),
                'nb_events': nb_events.flatten(),
                'nb_groups': nb_groups.flatten(),
                'ff_conf': ff_conf.flatten(),
            })
 
        #create shape file
        #ensure all values are of dtype float (e.g. in case there is a missing value!)
        dataset['lon']=dataset['lon'].astype(float)
        dataset['lat']=dataset['lat'].astype(float)
        dataset['flash_id']=dataset['flash_id'].astype(int)
        
        dataset['flash_time']=dataset['flash_time'].astype(str)
        dataset['flash_dur']=dataset['flash_dur'].astype(float)
        dataset['flash_fp']=dataset['flash_fp'].astype(float)       

        dataset['rad_scale']=dataset['rad_scale'].astype(float)
        dataset['nb_events']=dataset['nb_events'].astype(float)
        dataset['nb_groups']=dataset['nb_groups'].astype(float)
        dataset['ff_conf']=dataset['ff_conf'].astype(float)           
 
        
        # Create geometry column
        geometry = [Point(xy) for xy in zip(dataset['lon'], dataset['lat'])]

        # Create GeoDataFrame
        gdf = gpd.GeoDataFrame(dataset, geometry=geometry)
        
        progress['value'] = 80

        # Set the coordinate reference system (CRS), WGS84
        gdf.set_crs(epsg=4326, inplace=True)

        # Write to shapefile
        gdf.to_file(WorkDir+'/'+OutFileName+'.shp')

        #save the results to disk as a CSV file
        #dataset.to_csv(WorkDir+'/'+OutFileName + '.csv')

        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported MTG LI LFL image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 600, height = 373)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MTG_LI_LFL.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MTG Li LFL')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MTG LI LFL background:

LI Level 2 Lightning Flashes (LFL) contains LI flashes - MTG - 0 degree: The definition of a flash is shared by LI and GLM; collections of groups that are correlated in space and time within the two windows of 330 milliseconds (temporal window) and 16.5km (space window), respectively. Even if the definition of a flash is not uniform among all lightning location systems, the simple fact that a flash is a collection of groups/strokes correlated in space and time somewhat mitigates the differences in the way different types of lightning sensors interpret different lightning processes. This makes flash datasets of different lightning location systems more comparable than group/stroke datasets. See also: https://data.eumetsat.int/product/EO:EUM:DAT:0691

File name convertion: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--x-x--ARC-x_C_EUMT_*.zip
Number of segments: after unzipping 1 body segment
Temporal reolution: Every 10 minutes, starting from 00:00 UTC
Initial file format: NetCDF

Data is provided throught EUMETCast reception and EUMDAC (online). For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL) import routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Meteosat Third Generation Lightning Imager Level 2 Lightning Flashes (LFL) 
Typical filename: W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+LI-2-LFL--FD--x-x--ARC-x_C_EUMT_*.zip""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Input Filename:", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
button_filein.pack()
button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About MTG-LI-LFL", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())



