#Meteosat Third Generation High Resolution 1km import routine
#Import of all visible (VIS) channels

from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import ilwis
import shutil
import sys, os 
import glob
from threading import Thread
from satpy.scene import Scene
from satpy import find_files_and_readers
import hdf5plugin
import site
import numpy as np

#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0

        time_stamp = InFileName
        year = time_stamp[0:4]
        month = time_stamp[4:6]
        day = time_stamp[6:8]
        timehrUTC = time_stamp[8:10]
        timehr = time_stamp[0:10] #hourly resolution is assumed here
        
        folder = [folder for folder in site.getsitepackages() if os.path.exists(folder + r'/hdf5plugin/plugins')]
        if len(folder) > 0:
            hdf5pluginfolder = folder[0] + r'/hdf5plugin/plugins'
            print(hdf5pluginfolder)
            os.environ["HDF5_PLUGIN_PATH"] = hdf5pluginfolder
            
        progress['value'] = 10

        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        for filename in glob.glob(os.path.join(GNCDir+'/'+'W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+FCI-1C-RRAD-FDHSI-FD--CHK-BODY--DIS-NC4E_C_EUMT_'+(timehr)+'*.nc')):
            shutil.copy(filename, WorkDir)

        files = find_files_and_readers(base_dir=WorkDir, reader='fci_l1c_nc')

        #read scene using satpy
        scn = Scene(filenames=files)

        progress['value'] = 20

        #VIS
        #load values of spectral channels
        #VIS04
        scn.load(["vis_04"])
        values = scn['vis_04'].values
        #flip UD image
        rot = np.flipud(values)
        data_04 = np.array([rot]).flatten()

        #VIS_05
        scn.load(["vis_05"])
        values = scn['vis_05'].values
        #flip UD image
        rot = np.flipud(values)
        data_05 = np.array([rot]).flatten()
        
       
        progress['value'] = 40
        
        #VIS_06
        scn.load(["vis_06"])
        values = scn['vis_06'].values
        #flip UD image
        rot = np.flipud(values)
        data_06 = np.array([rot]).flatten()

        
        #VIS_08
        scn.load(["vis_08"])
        values = scn['vis_08'].values
        #flip UD image
        rot = np.flipud(values)
        data_08 = np.array([rot]).flatten()

        
        #VIS_09
        scn.load(["vis_09"])
        values = scn['vis_09'].values
        #flip UD image
        rot = np.flipud(values)
        data_09 = np.array([rot]).flatten()

        
        progress['value'] = 60

        
        data_fin = np.array((data_04, data_05, data_06, data_08, data_09))
        
        progress['value'] = 80  
        
        #export to ILWIS format
        #create an empty raster
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +h=35786400 +a=6378137 +rf=298.257223563,envelope=-5567999.9986 5567999.9986 5567999.9986 -5567999.9986,gridsize=11136 11136,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(-100.0, 500.0, 0.001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(11136,11136,5))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        #write data to raster
        rcNew.array2raster(data_fin)
            
        #correct background and save the results to disk as an ILWIS mpr file
        rcNew = ilwis.do('mapcalc', 'iff(@1>=-100,@1,?)', rcNew) # remove (satpy) nodata
        rcNew.store(OutFileName+'.mpl')
        
        rcNew = None
        data = None
        rot = None
        scn = None
        values = None


        progress['value'] = 100
        button0.config(state='normal')
        button1.config(state='normal')

#redundant files deleted in main menu script

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of imported MTG composite image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 400, height = 410)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/MTG_NR_fc.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About MTG_NR')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""MTG NR background:

FCI Level 1c Normal Resolution Image Data - MTG - 0 degree: The FCI in Normal Resolution (NR) has channels over 16 spectral ranges covering visible to infrared wavelengths. The spectral channels VIS 0.4, VIS 0,5 VIS 0.6, VIS 0.8, VIS 0.9, NIR 1.3, NIR 1.6, NIR 2.2 are delivered in a spatial resolution of 1 km sub-satellite, IR 3.8, WV 6.2, 7.3, IR 8.7, IR 9.7, IR 10.5, IR 12.3 and IR 13.3 are delivered in a spatial resolution of 2 km sub satellite. NR is also referred to as FDHSI (Full Disc High Spectral Resolution Imagery). FCI Level 1c rectified radiance dataset consists of a set of files that contain the level 1c science data rectified to a reference grid together with the auxiliary data associated with the processing configuration and the quality assessment of the dataset. Level 1c image data here corresponds to initially geolocated and radiometrically pre-processed image data, without full georeferencing and cal/val in spatial and spectral domains applied. See also: https://data.eumetsat.int/product/EO:EUM:DAT:0662

File name convention: W_XX-EUMETSAT-Darm*,IMG+SAT,MTI1+FCI-1C-RRAD-FDHSI-FD--CHK-BODY--DIS-NC4E_C_*.nc
Number of segments: 40 body segments FCI compressed
Temporal resolution: Every 10 minutes, starting from 00:00 UTC
Initial file format: NetCDF
Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("Meteosat Third Generation Normal Resolution VIS import routine")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""Meteosat Third Generation FCI Normal Resolution 1 km Channels Import Routine  
Typ.f_name:W_XX-EUMETSAT-Darmstadt,IMG+SAT,MTI1+FCI-1C-RRAD-FDHSI-FD--CHK-BODY--DIS-NC4E_C_E*.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename prefix:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About MTG-FCI-NR", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


