#NWCSAF 0-degree RDT import routine 
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob
import bz2

import ilwis
import netCDF4 as nc
from threading import Thread
import subprocess
import numpy as np


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp_in = InFileName
        timeA = time_stamp_in[:8]
        timeB = time_stamp_in[8:12]
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filenames expected =  S_NWC_RDT-CW_MSG3_global-VISIR_20250422T131500Z.nc
        for filename in glob.glob(os.path.join(GNCDir+'/S_NWC_RDT-CW_MSG3_global-VISIR_'+(timeA)+'T'+(timeB)+'00Z.nc')):
            shutil.copy(filename, WorkDir)
           
        progress['value'] =30
        
        #load CMA dataset
        fn = WorkDir+'/S_NWC_RDT-CW_MSG3_global-VISIR_'+(timeA)+'T'+(timeB)+'00Z.nc'
        ds = nc.Dataset(fn)
        ct = ds['MapCellCatType']


        
        #create numpy array from nc variable
        ct_np = ct[:].flatten()

      
        progress['value'] = 60

        #MSG window
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +h=35785831 +a=6378137 +rf=298.257223563,envelope=-5570248.4773 5570248.4773 5567248.0742 -5567248.0742,gridsize=3712 3712,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0.0, 1000.0, 0.000))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(3712,3712,1))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        progress['value'] = 80

        rcNew.array2raster(ct_np.flatten())
        rcNew.store(OutFileName+'_val.mpr')
        
        #comment on cell cat type= "0:Non_convective 1:Convective_triggering 2:Convective_triggering_from_split 3:Convective_growing 4:Convective_mature
        #5:OvershootingTop_mature 6:Convective_decaying 7:Electric_triggering 8:Electric_triggering_from_split 9:Electric_growing 10:Electric_mature 
        #11:Electric_decaying 12:HighRainRate_triggering 13:HighRainRate_triggering_from_split 14:HighRainRate_growing 15:HighRainRate_mature 
        #16:HighRainRate_decaying 17:HighSeverity_triggering 18:HighSeverity_triggering_from_split 19:HighSeverity_growing 20:HighSeverity_mature 
        #21:HighSeverity_decaying";
        
        # create a new NumericItemRange and fill it with required class item
        
        cltrange = ilwis.NumericItemRange()
        cltrange.add(('Non_convective', 0.0, 0.99)) #0
        cltrange.add(('Convective_triggering', 1.0, 1.99)) #1
        cltrange.add(('Convective_triggering_from_split', 2.0, 2.99)) #2
        cltrange.add(('Convective_growing', 3.0, 3.99)) #3
        cltrange.add(('Convective_mature', 4.0, 4.99)) #4
        cltrange.add(('OvershootingTop_mature', 5.0, 5.99)) #5
        cltrange.add(('Convective_decaying', 6.0, 6.99)) #6
        cltrange.add(('Electric_triggering', 7.0,7.99)) #7
        cltrange.add(('Electric_triggering_from_split', 8.0, 8.99)) #8
        cltrange.add(('Electric_growing', 9.0, 9.99)) #9
        cltrange.add(('Electric_mature', 10.0, 10.99)) #10
        cltrange.add(('Electric_decaying', 11.0, 11.99)) #11
        cltrange.add(('HighRainRate_triggering', 12.0, 12.99)) #12
        cltrange.add(('HighRainRate_triggering_from_split', 13.0, 13.99)) #13
        cltrange.add(('HighRainRate_growing', 14.0, 14.99)) #14
        cltrange.add(('HighRainRate_mature', 15.0,15.99)) #15
        cltrange.add(('HighRainRate_decaying', 16.0, 16.99)) #16
        cltrange.add(('HighSeverity_triggering', 17.0, 17.99)) #17
        cltrange.add(('HighSeverity_triggering_from_split ', 18.0, 18.99)) #18
        cltrange.add(('HighSeverity_growing', 19.0, 19.99)) #19
        cltrange.add((':HighSeverity_mature ', 20.0, 20.99)) #20
        cltrange.add(('HighSeverity_decaying', 21.0, 21.99)) #21

        #assign the range to a new ItemDomain
        clt_dom = ilwis.ItemDomain(cltrange)
        
        cloud_cl = ilwis.do('sliceraster', rcNew, clt_dom)
        
        cloud_cl.datadef().domain().name(OutFileName+'_cl') 
        
        cloud_cl.store(OutFileName+'_cl.mpr')
        
        for filename in glob.glob(os.path.join(gnc_path+'/extern/ilwis386_2system/NWC_RDT_clt_outClass.rp*')):
            #print(filename)
            shutil.copy(filename, WorkDir)  
        
        os.replace(WorkDir+'/NWC_RDT_clt_outClass.rpr', WorkDir+'/'+OutFileName+'_cl.rpr')
        os.replace(WorkDir+'/NWC_RDT_clt_outClass.rp#', WorkDir+'/'+OutFileName+'_cl.rp#')


        #input file
        fin = open(WorkDir+'/'+OutFileName+'_cl.rpr', 'rt')
        #output file to write the result to
        fout = open(WorkDir+'/temp.rpr', 'wt')
        #for each line in the input file
        for line in fin:
            #read replace the string and write to output file
            fout.write(line.replace('NWC_RDT_clt_outClass', OutFileName+'_cl'))
        #close input and output files
        fin.close()
        fout.close()

        #remove file
        if os.path.exists(WorkDir+'/'+OutFileName+'_cl.rpr'):
            os.remove(WorkDir+'/'+OutFileName+'_cl.rpr')
        #else:
        #    print("The file does not exist")


        old_name = WorkDir+'/temp.rpr'
        new_name = WorkDir+'/'+OutFileName+'_cl.rpr'
        # Renaming the file
        os.rename(old_name, new_name)
        
        #Other redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG @0 degree cloud type 15 minutes image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 400, height = 399)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/NWCSAF_MSG0_RDT.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About RDT')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""RDT background:

Rapidly Developing Thunderstorms - MSG - 0 degree. Rapidly Developing Thunderstorms - Convection Warning product is a geostationary meteorological product for nowcasting applications. Using mainly MSG satellite data, it provides information on clouds related to significant convective systems, from mesoscale (200 to 2000 km) down to smaller scales (tenths of km). The objectives of RDT are the identification, monitoring and tracking of intense convective system cloudsand the detection of rapidly developing convective cells. The object-oriented approach underlying the RDT product allows for adding value to the satellite image by characterizing convective, spatially consistent, entities through various parameters of interest to the users such as motion vector, cooling rate, cloud top height, etc. It is produced with the NWC-SAF Geo 2018.1 software package.

File name convertion: S_NWC_RDT-CW_MSG3_global-VISIR_20250423T071500Z.nc 
Number of segments: 1 file
Temporal resolution: 1 per 15 minutes
Initial file format: nc format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("NWC SAF RDT")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""NWC SAF RDT import routine   
Typical filename: S_NWC_RDT-CW_MSG3_global-VISIR_20250423T071500Z.nc """)

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About RDT", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


