#NWCSAF 45.5-degree CT import routine 
from tkinter import *
from tkinter.filedialog import askdirectory
from tkinter.filedialog import askopenfilename
from tkinter.ttk import Progressbar, Style
import os
import sys
import shutil
import glob
import bz2

import ilwis
import netCDF4 as nc
from threading import Thread
import subprocess
import numpy as np


#passing parameters from sub menu

def execute(GNCDir, InFileName, WorkDir, OutFileName):
    if len(GNCDir) > 0 and len(InFileName) > 0 and len(WorkDir) > 0 and len(OutFileName) > 0:

        progress['value'] = 0
        time_stamp_in = InFileName
        timeA = time_stamp_in[:8]
        timeB = time_stamp_in[8:12]
        
        progress['value'] = 10
        
        os.chdir(WorkDir)
        ilwis.setWorkingCatalog(WorkDir)
        
        #filenames expected =  S_NWC_CT_MSG2_IODC-VISIR_20250423T110000Z.nc
        for filename in glob.glob(os.path.join(GNCDir+'/S_NWC_CT_MSG2_IODC-VISIR_'+(timeA)+'T'+(timeB)+'00Z.nc')):
            shutil.copy(filename, WorkDir)
           
        progress['value'] =30
        
        #load CT dataset
        fn = WorkDir+'/S_NWC_CT_MSG2_IODC-VISIR_'+(timeA)+'T'+(timeB)+'00Z.nc'
        ds = nc.Dataset(fn)
        ct = ds['ct'] #comment = "1:  Cloud-free land; 2:  Cloud-free sea; 3:  Snow over land;  4:  Sea ice; 5:  Very low clouds; 6:  Low clouds; 7:  Mid-level clouds;  8:  High opaque clouds; 9:  Very high opaque clouds;  10:  Fractional clouds; 11:  High semitransparent thin clouds;  12:  High semitransparent moderately thick clouds;  13:  High semitransparent thick clouds;  14:  High semitransparent above low or medium clouds;  15:  High semitransparent above snow/ice";
        #ct_cumuliform = ds['ct_cumuliform'] #comment = "1:  Stratiform status; 2:  Cumuliform status; 3:  Mixed status; 4:  Cloud-free; 5:  Undefined (separability problems)";
        #ct_multilayer = ds['ct_multilayer'] #comment = "0:  No multilayer detected; 1:  Multilayer detected;  2:  Cloud free; 3:  Undefined (separability problems)";

        
        #create numpy array from nc variable
        ct_np = ct[:].flatten()
        #ct_cumuliform_np = ct_cumuliform[:].flatten()
        #ct_multilayer_np = ct_multilayer[:].flatten()
      
        progress['value'] = 60

        #MSG IODC window
        grf = ilwis.GeoReference('code=georef:type=corners,csy=proj4:+proj=geos +lon_0=45.5 +h=35785831 +a=6378137 +rf=298.257223563,envelope=-5570248.4773 5570248.4773 5567248.0742 -5567248.0742,gridsize=3712 3712,cornerofcorners=yes')
        dfNum = ilwis.DataDefinition(ilwis.NumericDomain('code=value'), ilwis.NumericRange(0.0, 1000.0, 0.001))
        rcNew = ilwis.RasterCoverage()
        rcNew.setSize(ilwis.Size(3712,3712,1))
        rcNew.setGeoReference(grf)
        rcNew.setDataDef(dfNum)
        
        progress['value'] = 80
        
        data = np.array((ct_np))#, ct_cumuliform_np, ct_multilayer_np))

        rcNew.array2raster(data.flatten())
        rcNew1 = ilwis.do('mapcalc', 'iff(@1<255,@1,?)', rcNew)
        rcNew.store(OutFileName+'_val.mpr')
        
        # create a new NumericItemRange and fill it with required class items
        cltrange = ilwis.NumericItemRange()
        cltrange.add(('Cloud-free land', 1.0, 1.99)) #1
        cltrange.add(('Cloud-free sea', 2.0, 2.99)) #2
        cltrange.add(('Snow over land', 3.0, 3.99)) #3
        cltrange.add(('Sea ice', 4.0, 4.99)) #4
        cltrange.add(('Very low clouds', 5.0, 5.99)) #5
        cltrange.add(('Low clouds', 6.0, 6.99)) #6
        cltrange.add(('Mid-level clouds', 7.0, 7.99)) #7
        cltrange.add(('High opaque clouds', 8.0, 8.99)) #8
        cltrange.add(('Very high opaque clouds', 9.0, 9.99)) #9
        cltrange.add(('Fractional clouds', 10.0, 10.99)) #10
        cltrange.add(('High semitransparent thin clouds', 11.0, 11.99)) #11
        cltrange.add(('High semitransparent meanly thick clouds', 12.0, 12.99)) #12
        cltrange.add(('High semitransparent thick clouds', 13.0, 13.99)) #13
        cltrange.add(('High semitransparent above low or medium clouds', 14.0, 14.99)) #14
        cltrange.add(('High semitransparent above snow/ice', 15.0, 15.99)) #15


        #assign the range to a new ItemDomain
        clt_dom = ilwis.ItemDomain(cltrange)
        
        cloud_cl = ilwis.do('sliceraster', rcNew, clt_dom)
        
        cloud_cl.datadef().domain().name(OutFileName+'_cl') 
        
        cloud_cl.store(OutFileName+'_cl.mpr')
        
        for filename in glob.glob(os.path.join(gnc_path+'/extern/ilwis386_2system/NWC_CTClass.rp*')):
            #print(filename)
            shutil.copy(filename, WorkDir)  
        
        os.replace(WorkDir+'/NWC_CTClass.rpr', WorkDir+'/'+OutFileName+'_cl.rpr')
        os.replace(WorkDir+'/NWC_CTClass.rp#', WorkDir+'/'+OutFileName+'_cl.rp#')


        #input file
        fin = open(WorkDir+'/'+OutFileName+'_cl.rpr', 'rt')
        #output file to write the result to
        fout = open(WorkDir+'/temp.rpr', 'wt')
        #for each line in the input file
        for line in fin:
            #read replace the string and write to output file
            fout.write(line.replace('NWC_CTClass', OutFileName+'_cl'))
        #close input and output files
        fin.close()
        fout.close()

        #remove file
        if os.path.exists(WorkDir+'/'+OutFileName+'_cl.rpr'):
            os.remove(WorkDir+'/'+OutFileName+'_cl.rpr')
        #else:
        #    print("The file does not exist")


        old_name = WorkDir+'/temp.rpr'
        new_name = WorkDir+'/'+OutFileName+'_cl.rpr'
        # Renaming the file
        os.rename(old_name, new_name)
              

        #Other redundant files are deleted in main menu script
        progress['value'] = 100
        
        button0.config(state='normal')
        button1.config(state='normal')

def executeInThread(GNCDir, InFileName, WorkDir, OutFileName):
    button0.config(state='disabled')
    button1.config(state='disabled')
    thread = Thread(target = execute, args = (GNCDir, InFileName, WorkDir, OutFileName))
    thread.start()



# for command=doNothing
def doNothing():
   text="Do nothing button"
   return


# for command=clickthumbn
def clickthumbn():
   novi = Toplevel(app)
   novi.title ('Example of visualized MSG @45.5 degree cloud type 15 minutes image (in ILWIS)')
   novi.iconbitmap(gnc_path +r'/pics/gnc.ico')
   canvas = Canvas(novi, width = 475, height = 400)
   canvas.pack(expand = YES, fill = BOTH)
   gif1 = PhotoImage(file = gnc_path +r'/pics/NWCSAF_MSG455_CloudType.gif')
   canvas.create_image(0, 0,image = gif1, anchor = NW)
   canvas.gif1 = gif1

# for command=clickAbout
def clickAbout(): 
   filewin = Toplevel(app)
   filewin.title ('About CT')
   filewin.iconbitmap(gnc_path +r'/pics/gnc.ico')
   filewin.geometry("575x340")
   msg = Message(filewin, text="""CT background:

Geostationary Nowcasting Cloud Type - MSG - Indian Ocean. The Indian Ocean Cloud Type is a geostationary meteorological product for nowcasting applications, covering the MSG - IODC disk. It is produced with the NWC-SAF Geo 2018 software package. From 1 June 2022, Meteosat-9 at 45.5° E is the prime satellite for the IODC service. The Cloud Type (CT) product allocates a meteorological cloud type to the identified clouds, gives indications if clouds are more Stratiform or Cumuliform and if Multilayer clouds are present respectively.

File name convertion: S_NWC_CT_MSG2_IODC-VISIR_20250423T110000Z.nc
Number of segments: 1 file
Temporal resolution: 1 per 15 minutes
Initial file format: nc format

Data is provided throught EUMETCast reception. For Legal Constraints check the 'Product Navigator' at EUMETSAT available at http://navigator.eumetsat.int
""")
   msg.pack()


#main()
#automatically setting of path when running this script from main menu - uncomment to make active   
gnc_path = os.environ["gnc_popup"]

#check path
print (gnc_path)
app = Tk()
app.title("NWC SAF CT")
app.iconbitmap(gnc_path +r'/pics/gnc.ico')

w = 650 # width for the Tk app
h = 210 # height for the Tk app

app.geometry('%dx%d' % (w, h))
app.resizable(0,0)

labelText1 = StringVar()
labelText1.set ("""NWC SAF CT import routine   
Typical filename: S_NWC_CT_MSG2_IODC-VISIR_20250423T110000Z.nc""")

label1 = Label(app, textvariable=labelText1, anchor='w', justify = LEFT)
label1.pack(side = LEFT)
label1.place(x = 10, y = 5, width=650, height=35)

L1 = Label(app, text="Input Directory:", anchor='w')
L1.pack(side = LEFT)
L1.place(x = 10, y = 50, width=150, height=25)

inputDir = StringVar()
E1 = Entry(app, textvariable=inputDir, bd =2)
E1.pack()
E1.place(x = 165, y = 50, width=445, height=25)

button_dirin = Button(app, text="....", width=2, command = lambda: inputDir.set(askdirectory(initialdir = inputDir.get())))
button_dirin.pack()
button_dirin.place(x = 615, y = 49)

L2 = Label(app, text="Time Stamp (in UTC-time):", anchor='w')
L2.pack(side = LEFT)
L2.place(x = 10, y = 80, width=150, height=25)

inputFile = StringVar()
E2 = Entry(app, textvariable=inputFile, bd =2)
E2.pack()
E2.place(x = 165, y = 80, width=445, height=25)

#button_filein = Button(app, text="....", width=2, command = lambda: inputFile.set(askopenfilename(initialdir = inputDir.get()).split('/')[-1]))
#button_filein.pack()
#button_filein.place(x = 615, y = 79)

L3 = Label(app, text="Output Directory:", anchor='w')
L3.pack(side = LEFT)
L3.place(x = 10, y = 110, width=150, height=25)

outputDir = StringVar()
E3 = Entry(app, textvariable=outputDir, bd =2)
E3.pack()
E3.place(x = 165, y = 110, width=445, height=25)

button_dirout = Button(app, text="....", width=2, command = lambda: outputDir.set(askdirectory(initialdir = outputDir.get())))
button_dirout.pack()
button_dirout.place(x = 615, y = 109) 

L4 = Label(app, text="Output Filename:", anchor='w')
L4.pack(side = LEFT)
L4.place(x = 10, y = 140, width=150, height=25)

outputFile = StringVar()
E4 = Entry(app, textvariable=outputFile, bd =2)
E4.pack()
E4.place(x = 165, y = 140, width=445, height=25)

button0 = Button(app, text="Quit", width=15, command=app.destroy)
button0.pack()
button0.place(x = 525, y = 180)

button1 = Button(app, text="Execute", width=15, command=lambda: executeInThread(inputDir.get(), inputFile.get(), outputDir.get(), outputFile.get()))
button1.pack()
button1.place(x = 405, y = 180)

button2 = Button(app, text="About CT", width=15, command=clickAbout)
button2.pack()
button2.place(x = 285, y = 180)

button3 = Button(app, text="Thumbnail", width=15, command=clickthumbn)
button3.pack()
button3.place(x = 165, y = 180)


style = Style()
style.theme_use('clam')
style.configure('blue.Horizontal.TProgressbar', foreground='blue', background='blue')

progress = Progressbar(app, style='blue.Horizontal.TProgressbar', orient=HORIZONTAL, length=100, mode='determinate')
progress.pack()
progress.place(x = 10, y = 180)


app.mainloop()

print("inputFile=" + inputFile.get())
print("outputDir=" + outputDir.get())
print("outputFile=" + outputFile.get())


